; /**
;   ******************************************************************************
;   *  _ _ _    _ _ _   _ _
;   * |     \  /     |/     \
;   * |      \/           _ _\
;   * |                  /
;   * |         MultiClet
;   * |                  \ _ _
;   * |                      /
;   * |_ _ _|\/|_ _ _|\ _ _ /
;   *
;   * @file    uart.asm 
;   * @author  Команда разработчиков Мультиклет
;   * @date    13-07-2015
;   * @brief   Пример работы с UART. 
;   *          
;   ******************************************************************************
;   * @attention
;   *
;   * <h2><center>&copy; COPYRIGHT(c) 2015 Multiclet corp.</center></h2>
;   *
;   *  ДАННЫЙ ПРИМЕР НАПИСАН ДЛЯ БЫСТРОГО ОЗНАКОМЛЕНИЯ ПОЛЬЗОВАТЕЛЯ С
;   *  ВОЗМОЖНОСТЯМИ МУЛЬТИКЛЕТОЧНОГО ПРОЦЕССОРА. КОМПАНИЯ МУЛЬТИКЛЕТ НЕ НЕСЁТ
;   *  ОТВЕТСТВЕННОСТИ ЗА ВОЗМОЖНЫЙ ПРЯМОЙ ИЛИ КОСВЕННЫЙ УЩЕРБ, ПРИЧИНЁННЫЙ 
;   *  ИСПОЛЬЗОВАНИЕМ В ПОЛЬЗОВАТЕЛЬСКИХ ПРОДУКТАХ ИНФОРМАЦИИ, СОДЕРЖАЩЕЙСЯ
;   *  В ДАННОМ ПРИМЕРЕ.
;   * 
;   ******************************************************************************
;   */
;файл констант периферии для ассемблера
.include "R1_pcf.inc"

.alias UART_BDR UART3_BDR
.alias UART_CR UART3_CR
.alias UART_ST UART3_ST
.alias UART_DATA UART3_DATA

.text

;расположение светодиодов: 
;LED0 - GPIOB(1<<25), LED1 - GPIOB(1<<24), LED2 - GPIOB(1<<23)
;LED3 - GPIOB(1<<27), LED4 - GPIOC(1<<26), LED5 - GPIOB(1<<29)
;LED6 - GPIOB(1<<28), LED7 - GPIOA(1<<25)
;расположение UART:
;UART3[TXD] - GPIOF(1<<14), UART3[RXD] - GPIOF(1<<13)
;UART1[TXD] - GPIOA(1<<23), UART1[RXD] - GPIOA(1<<22), UART1[CTS] - GPIOA(1<<24), UART1[RTS] - GPIOA(1<<25)
;UART2[TXD] - GPIOC(1<<11), UART2[RXD] - GPIOC(1<<10)
;UART0[TXD] - GPIOC(1<<22), UART0[RXD] - GPIOC(1<<21)


;сконфигурируем порт для LED1 на выход и для UART3 на альтернативную функцию
config_port:
    rdl GPIOB_DIR 
    orl @1, (1<<24) 
    wrl @1, GPIOB_DIR ;настраиваем порт B на выход для вывода 24
    rdl GPIOF_BPS
    orl @1, (1<<14)|(1<<13)
    wrl @1, GPIOF_BPS ;включаем альтернативную функцию порта F для UART3
    jmp uart_config
complete

;настройка UART3
uart_config:
    getl 0x1A ;значение для UART_BDR рассчитывается по формуле: BRDIV = Fsys/(Fuart*8-1)
    wrl @1, UART_BDR
    getl 0x3 ;разрешаем работу RX, TX
    wrl @1, UART_CR
    jmp fill_data
complete

;заполнение РОНа данными и установка счетчика
fill_data:
    getl 0x12345678
    getl 0xABCDEF11
    patch @1, @2 ;сформируем 64-х разрядное значение
    setq #0, @1 ;запишем значение в РОН 0
    setl #1, 0 ;установим счетчик для вывода 64-х бит
    jmp uart_tx_ready
complete

uart_tx_ready:
    rdl UART_ST
    getl 1<<9
    andl @1, @2
    jne @1, uart_tx_ready
    je @2, uart_send_byte
complete

uart_send_byte:
        getq #0
cnt:=   getl #1 ;запись cnt:= использована для удобства ссылки
        slrq @2, @1 ;сдвигаем вправо по счетчику
        wrb @1, UART_DATA
        addl @cnt, 8 ;увеличиваем счетчик на 8
        setl #1, @1 ;записываем значение в РОН 1
        subl @cnt, 56 ;проверяем отправку последнего байта
        je @1, led1_on
        jne @2, uart_tx_ready
complete

;включение светодиода
led1_on:
    rdl GPIOB_OUT ;считываем текущее значение порта в регистр передаваемых данных порта В
    orl @1, (1<<24) ;формируем значение для включения логической единицы на 24-м разряде порта B
    wrl @1, GPIOB_OUT ;записываем получившееся значение в регистр передаваемых данных порта B
complete




