/****************************************Copyright (c)****************************************************
**                                      
**                                 http://www.powermcu.com
**
**--------------File Info---------------------------------------------------------------------------------
** File name:               TouchPanel.c
** Descriptions:            The TouchPanel application function
**
**--------------------------------------------------------------------------------------------------------
** Created by:              AVRman
** Created date:            2010-11-7
** Version:                 v1.0
** Descriptions:            The original version
**
**--------------------------------------------------------------------------------------------------------
** Modified by:             Multiclet team
** Modified date:           
** Version:                 
** Descriptions:            
**
*********************************************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "GLCD.h"
#include <stdint.h>
#include "TouchPanel.h"
#include "GLCD.h"


/*******************************************************************************
* Function Name  : TP_Init
* Description    : ADS7843 SPI Initialization
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
int TP_Init(void) 
{ 
	*((uint32_t*)0xC0108020)=(*((uint32_t*)0xC0108020))&(~0x1000000);
	*((uint32_t*)0xC0108020)=(*((uint32_t*)0xC0108020))&(~0x10000000);

	
	GPIOF->BPS &= ~0xE0000;
		
	
	*((uint32_t*)0xC0108038)=1;
	*((uint32_t*)0xC0108020)=0x77F2085&0xFEFFFFFF;
		
	*((uint32_t*)0xC0108020)=(*((uint32_t*)0xC0108020))|0x1000000;
	
	TP_CS(1); 
	return 0;
} 

/*******************************************************************************
* Function Name  : DelayUS
* Description    : delay 1us
* Input          : - cnt: delay value
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
int DelayUS( unsigned int cnt)
{
  int i,j;
	for(i=0;i<cnt;i++){		
		for(j=0;j<0x2A8;j++);
	}
	return 0;
}

/*******************************************************************************
* Function Name  : WR_CMD
* Description    : write data to ADS7843
* Input          : - cmd: data
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
uint8_t WR_CMD (uint8_t cmd)  
{ 	
	/* wait for current SPI activity complete */
	while (((*((uint32_t*)0xC0108024))&0x100)==0);


        /* Send SPI data */ 
	*((uint32_t*)0xC0108030)=cmd<<24;


        /* Wait for SPI data reception */ 
	while (((*((uint32_t*)0xC0108024))&0x80000000));
	while (((*((uint32_t*)0xC0108024))&0x200)==0);

        /* Read SPI1 received data */
	return (*((uint32_t*)0xC0108034))>>16;
} 

/*******************************************************************************
* Function Name  : RD_AD
* Description    : read adc value
* Input          : None
* Output         : None
* Return         : ADS7843 Return two byte data
* Attention		 : None
*******************************************************************************/

int RD_AD(void)  
{ 
	unsigned short buf,temp; 
	
	temp = WR_CMD(0x00);

	buf = temp << 4; 
	DelayUS(1); 

	temp = WR_CMD(0x00);

	buf |= temp>>4; 
	buf &= 0xfff;

	return buf; 
} 

/*******************************************************************************
* Function Name  : Read_X
* Description    : Read ADS7843 ADC value of X + channel 
* Input          : None
* Output         : None
* Return         : return X + channel ADC value
* Attention		 : None
*******************************************************************************/

int Read_X(void)  
{  
	int i; 
	TP_CS(0); 
	DelayUS(1); 
	WR_CMD(CHX); 
	DelayUS(1); 
	i = RD_AD(); 
	TP_CS(1); 
	return i;    
} 

/*******************************************************************************
* Function Name  : Read_Y
* Description    : Read ADS7843 ADC value of Y + channel 
* Input          : None
* Output         : None
* Return         : return Y + channel ADC value
* Attention		 : None
*******************************************************************************/

int Read_Y(void)  
{  
	int i; 
	TP_CS(0); 
	DelayUS(1); 
	WR_CMD(CHY); 
	DelayUS(1); 
	i = RD_AD(); 
	TP_CS(1); 
	return i;     
} 

/*******************************************************************************
* Function Name  : TP_GetAdXY
* Description    : Read ADS7843 ADC value of X + Y + channel 
* Input          : None
* Output         : None
* Return         : return X + Y + channel ADC value
* Attention		 : None
*******************************************************************************/

int TP_GetAdXY(int *x,int *y)  
{ 
	int adx,ady; 
	adx = Read_X(); 
	DelayUS(1); 
	ady = Read_Y(); 
	*x = adx; 
	*y = ady; 
	return 0;
} 

/*******************************************************************************
* Function Name  : TP_DrawPoint
* Description    : Draw point Must have a LCD driver
* Input          : - Xpos: Row Coordinate
*                  - Ypos: Line Coordinate 
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
int TP_DrawPoint(uint16_t Xpos,uint16_t Ypos)
{
  LCD_SetPoint(Xpos,Ypos,0xf800);     /* Center point */
  LCD_SetPoint(Xpos+1,Ypos,0xf800);
  LCD_SetPoint(Xpos,Ypos+1,0xf800);
  LCD_SetPoint(Xpos+1,Ypos+1,0xf800);	
  return 0;
}

/*******************************************************************************
* Function Name  : Read_Ads7846
* Description    : X Y obtained after filtering
* Input          : None
* Output         : None
* Return         : Coordinate Structure address
* Attention		 : None
*******************************************************************************/
int Read_Ads(int * X,int * Y){	
  int x,y,i=0;
  long bx=0,by=0;
  *X=0;*Y=0;
	while(i<32){			   
	TP_GetAdXY(&y,&x); 	
	if((GPIOF->IN&0x20000)==0){
		by+=y;
		bx+=x;
		i++;
	}
	else{	
		DelayUS(2);
		if((GPIOF->IN&0x20000)!=0)
			return 1;
	}
  }
  *X=bx>>5;
  *Y=by>>5;
  DelayUS(2);
  return 0;
}
/*********************************************************************************************************
      END FILE
*********************************************************************************************************/
